enum LogLevel {
  LEVEL_VERBOSE,
  LEVEL_DEBUG,
  LEVEL_INFO,
  LEVEL_WARNING,
  LEVEL_ERROR,
  LEVEL_SILENCE,
}

enum IOTCAuthenticationType {
  AUTHENTICATE_BY_KEY,
}

enum AvSecurityMode {
  AV_SECURITY_SIMPLE,
  AV_SECURITY_DTLS,
  AV_SECURITY_AUTO,
}

enum AvAuthType {
  AV_AUTH_PASSWORD,
  AV_AUTH_TOKEN,
  AV_AUTH_NEBULA,
}

enum AV_RESET_TARGET {
  RESET_VIDEO,
  RESET_AUDIO,
  RESET_ALL,
}

interface LogAttr {
  log_level: LogLevel;
  path: string;
  file_max_size: number;
  file_max_count: number;
}

interface LanSearchInfo {
  UID: string;
  IP: string;
  port: number;
}

interface IOTCConnectInput {
  authentication_type: IOTCAuthenticationType;
  auth_key: string;
  device_region: string;
  timeout: number;
}

interface SInfoEx {
  Mode: number;
  CorD: number;
  UID: string;
  RemoteIP: string;
  RemotePort: number;
  TX_Packetcount: number;
  RX_Packetcount: number;
  IOTCVersion: number;
  VID: number;
  PID: number;
  GID: number;
  isSecure: number;
  LocalNatType: number;
  RemoteNatType: number;
  RelayType: number;
  NetState: number;
  RemoteWANIP: string;
  RemoteWANPort: number;
  isNebula: number;
  isUseAuthkey: number;
}

interface SearchDeviceInfo {
  UID: string;
  IP: string;
  port: number;
  DeviceName: string;
  nNew: number;
}

interface AVClientStartInConfig {
  iotc_session_id: number;
  iotc_channel_id: number;
  timeout_sec: number;
  account_or_identity: string;
  password_or_token: string;
  resend: number;
  security_mode: AvSecurityMode;
  auth_type: AvAuthType;
  sync_recv_data: number;
  dtls_cipher_suites: string;
}

interface AVClientStartOutConfig {
  server_type: number;
  resend: number;
  two_way_streaming: number;
  sync_recv_data: string;
  security_mode: AvSecurityMode;
}

interface avPasswordAuthFn {
  (account: string, password: string[], password_buf_size: number): number;
}

interface avTokenAuthFn {
  (identity: string, token: string[], token_buf_size: number): number;
}

interface avTokenRequestFn {
  (av_index: number, identity: string, identity_description: string, token: string[], token_buf_size: number): number;
}

interface avTokenDeleteFn {
  (av_index: number, identity: string): number;
}

interface avIdentityArrayRequestFn {
  (av_index: number, identities: string[]): number;
}

interface avAbilityRequestFn {
  (av_index: number, ability: string[]): number;
}

interface avChangePasswordRequestFn {
  (av_index: number, account: string, old_password: string, new_password: string, new_iotc_authkey: string): number;
}

interface avRecvJSONCtrlRequestFn {
  (av_index: number, func: string, json_args: string, json_response: string[]): number;
}

interface AVServStartInConfig {
  iotc_session_id: number;
  iotc_channel_id: number;
  timeout_sec: number;
  server_type: number;
  resend: number;
  security_mode: AvSecurityMode;
  password_auth: avPasswordAuthFn;
  token_auth: avTokenAuthFn;
  token_request: avTokenRequestFn;
  token_delete: avTokenDeleteFn;
  identity_array_request: avIdentityArrayRequestFn;
  ability_request: avAbilityRequestFn;
  change_password_request: avChangePasswordRequestFn;
  json_request: avRecvJSONCtrlRequestFn;
  dtls_cipher_suites: string;
}

interface AVServStartOutConfig {
  resend: number;
  two_way_streaming: number;
  auth_type: AvAuthType;
  account_or_identity: string;
}

export const TUTK_SDK_Set_License_Key: (key: string) => number;
export const TUTK_SDK_Set_Region_Code: (region: string) => number;
export const TUTK_Set_Log_Attr: (attr: LogAttr) => number;
export const IOTC_Set_Log_Attr: (attr: LogAttr) => number;
export const IOTC_Setup_Session_Alive_Timeout: (timeout: number) => number;
export const IOTC_Initialize2: (udpPort: number) => number;
export const IOTC_DeInitialize: () => number;
export const IOTC_Get_Version_String: () => string;
export const IOTC_Lan_Search: (info: LanSearchInfo[], arrayLen: number, waitTimeMs: number) => number;
export const IOTC_Connect_Stop_BySID: (sid: number) => number;
export const IOTC_Session_Close: (sid: number) => number;
export const IOTC_Get_SessionID: () => number;
export const IOTC_Connect_ByUIDEx: (uid: string, sid: number, input: IOTCConnectInput) => number;
export const IOTC_Connect_ByUID_Parallel: (uid: string, sid: number) => number;
export const IOTC_Session_Check_Ex: (sid: number, info: SInfoEx) => number;
export const IOTC_Session_Get_Free_Channel: (sid: number) => number;
export const IOTC_Session_Channel_OFF: (sid: number, channelId: number) => number;
export const IOTC_WakeUp_Setup_Auto_WakeUp: (enable: number) => number;
export const IOTC_Search_Device_Start: (waitTimeMs: number, sendIntervalMs: number) => number;
export const IOTC_Search_Device_Result: (info: SearchDeviceInfo[], arrayLen: number, getAll: number) => number;
export const IOTC_Search_Device_Stop: () => number;
export const AV_Set_Log_Attr: (attr: LogAttr) => number;
export const avInitialize: (maxChannelNum: number) => number;
export const avDeInitialize: () => number;
export const avGetAVApiVersionString: () => string;
export const avClientRequestChangeServerPassword: (avIndex: number, account: string, oldPassword: string, newPassword: string, newIotcAuthKey: string [], timeoutSec: number) => number;
export const avClientStartEx: (inConfig: AVClientStartInConfig, outConfig: AVClientStartOutConfig) => number;
export const avClientSetRecvBufMaxSize: (channelId: number, maxBufSizeKB: number) => number;
export const avSendIOCtrlExit: (avChannelId: number) => number;
export const avClientStop: (avChannelId: number) => number;
export const avClientExit: (sid: number, iotcChannelId: number) => number;
export const avServStop: (avChannelId: number) => number;
export const avServExit: (sid: number, iotcChannelId: number) => number;
export const avSendAudioData: (avChannelId: number, audioData: Uint8Array, frameInfo: Uint8Array) => number;
export const avSendIOCtrl: (avChannelId: number, ioctrlType: number, ioctrlData: Uint8Array) => number;
export const avRecvIOCtrl: (avChannelId: number, ioctrlType: number[], ioctrlData: Uint8Array, timeout: number) => number;
export const avClientCleanVideoBuf: (avChannelId: number) => number;
export const avRecvFrameData2: (avChannelId: number, frameData: Uint8Array, actualFrameSize: number[], expectedFrameSize: number[], frameInfo: Uint8Array, actualFrameInfoSize: number[], frameIdx: number[]) => number;
export const avServStartEx: (inConfig: AVServStartInConfig, outConfig: AVServStartOutConfig) => number;
export const avServSetResendSize: (avChannelId: number, size: number) => number;
export const avSendFrameData: (avChannelId: number, frameData: Uint8Array, frameInfo: Uint8Array) => number;
export const avServResetBuffer: (avIndex: number, target: AV_RESET_TARGET, timeoutMs: number) => number;
export const avClientCleanAudioBuf: (avChannelId: number) => number;
export const avSendJSONCtrlRequest: (avIndex: number, jsonRequest: string, jsonResponse: string[], timeoutSec: number) => number;
